package main

import (
	"git.beejay.kim/Gshopper/sentio"
	"git.beejay.kim/Gshopper/sentio/indicator"
	"git.beejay.kim/Gshopper/sentio/util"
	"time"
)

const (
	ATR_MULTIPLIER = 3
	ATR_PERIOD     = 21
	ATR_HHV        = 10
)

var Strategy = qqq{}

type qqq struct{}

func (strategy qqq) Name() string {
	return "Alpaca: QQQ [TQQQ : SQQQ]"
}

func (strategy qqq) Model() string {
	return "qqq15"
}

func (strategy qqq) MarketId() string {
	return "alpaca"
}

func (strategy qqq) Interval() uint8 {
	return 5
}

func (strategy qqq) PositionSymbols() map[sentio.Side]string {
	return map[sentio.Side]string{
		sentio.BASE:  "QQQ",
		sentio.LONG:  "TQQQ",
		sentio.SHORT: "SQQQ",
	}
}

func (strategy qqq) PositionProbabilities() map[sentio.Side]float64 {
	return map[sentio.Side]float64{
		sentio.BASE:  -1,
		sentio.LONG:  1.0008,
		sentio.SHORT: .9990,
	}
}

func (strategy qqq) Handle(market sentio.Market, probability sentio.Probability) error {
	var (
		now        = market.Clock().Now()
		symbols    = util.Symbols(strategy)
		stoplosses map[string]float64
		quotes     map[string]sentio.Quote
		orders     []sentio.Order
		err        error
	)

	// skip too early orders
	if now.Hour() == 9 && now.Minute() < 45 {
		return nil
	}

	// close all orders before market close
	if now.Hour() == 15 && now.Minute() > 50 {
		return util.CloseAllOrders(market, strategy)
	}

	// retrieve running orders
	if orders, err = market.Orders(sentio.OrderListCriteria{
		Status:  "open",
		Symbols: symbols,
	}); err != nil {
		return err
	}

	if stoplosses, err = indicator.AtrTrailingStopLoss(market, 1, ATR_PERIOD, ATR_MULTIPLIER, ATR_HHV, symbols...); err != nil {
		return err
	}

	// update stoplosses or close running orders
	hasClosedOrders := false
	for i := range orders {
		if strategy.PositionSymbols()[sentio.LONG] == orders[i].GetSymbol() &&
			probability.Value < 1.0008 {
			if _, err = market.CloseOrder(orders[i]); err != nil {
				return err
			}

			hasClosedOrders = true
			continue
		}

		if strategy.PositionSymbols()[sentio.SHORT] == orders[i].GetSymbol() &&
			probability.Value > .9998 {
			if _, err = market.CloseOrder(orders[i]); err != nil {
				return err
			}

			hasClosedOrders = true
			continue
		}

		if f, ok := stoplosses[orders[i].GetSymbol()]; ok && f > 0 {
			if err = market.UpdateOrder(orders[i].GetId(), f); err != nil {
				return err
			}

			continue
		}
	}

	// Prevent new orders if we just closed one
	if hasClosedOrders {
		return nil
	}

	if probability.TS.Add(time.Minute * time.Duration(int64(strategy.Interval()/2))).Before(now.Round(time.Minute)) {
		return nil
	}

	// Prevent BUYs on closing market
	if now.Hour() == 15 && now.Minute() > 46 {
		return nil
	}

	// Prevent Market.CreateOrder while probability is not clear
	if probability.Value == 1 || probability.Value < 0 {
		return nil
	}

	if quotes, err = market.Quotes(symbols...); err != nil {
		return err
	}

	var t = sentio.SHORT
	if probability.Value > 1 {
		t = sentio.LONG
	}

	return util.CreateOrder(market, strategy, t, probability, quotes, stoplosses)
}
